@@ -31,7 +31,7 @@
 #define NPRIORITIES 2
 
 struct cmdline_saved_param {
-    char *p, *value;
+    CmdlineArg *p, *value;
 };
 struct cmdline_saved_param_set {
     struct cmdline_saved_param *params;
@@ -44,11 +44,11 @@ struct cmdline_saved_param_set {
  */
 static struct cmdline_saved_param_set saves[NPRIORITIES];
 
-static void cmdline_save_param(const char *p, const char *value, int pri)
+static void cmdline_save_param(CmdlineArg *p, CmdlineArg *value, int pri)
 {
     sgrowarray(saves[pri].params, saves[pri].savesize, saves[pri].nsaved);
-    saves[pri].params[saves[pri].nsaved].p = dupstr(p);
-    saves[pri].params[saves[pri].nsaved].value = dupstr(value);
+    saves[pri].params[saves[pri].nsaved].p = p;
+    saves[pri].params[saves[pri].nsaved].value = value;
     saves[pri].nsaved++;
 }
 
@@ -73,7 +73,7 @@ void cmdline_cleanup(void)
 }
 
 #define SAVEABLE(pri) do { \
-    if (need_save) { cmdline_save_param(p, value, pri); return ret; } \
+    if (need_save) { cmdline_save_param(arg, nextarg, pri); return ret; }   \
 } while (0)
 
 /*
@@ -130,10 +130,15 @@ SeatPromptResult cmdline_get_passwd_input(
     return SPR_OK;
 }
 
+static void cmdline_report_unavailable(const char *p)
+{
+    cmdline_error("option \"%s\" not available in this tool", p);
+}
+
 static bool cmdline_check_unavailable(int flag, const char *p)
 {
     if (cmdline_tooltype & flag) {
-        cmdline_error("option \"%s\" not available in this tool", p);
+        cmdline_report_unavailable(p);
         return true;
     }
     return false;
@@ -185,10 +190,13 @@ static void set_port(Conf *conf, int port)
     conf_set_int(conf, CONF_port, port);
 }
 
-int cmdline_process_param(const char *p, char *value,
+int cmdline_process_param(CmdlineArg *arg, CmdlineArg *nextarg,
                           int need_save, Conf *conf)
 {
     int ret = 0;
+    const char *p = cmdline_arg_to_str(arg);
+    const char *value_utf8 = cmdline_arg_to_utf8(nextarg);
+    const char *value = cmdline_arg_to_str(nextarg);
 
     if (p[0] != '-') {
         if (need_save < 0)
@@ -403,9 +411,8 @@ int cmdline_process_param(const char *p, char *value,
              * pretend we received a -P argument, so that it will be
              * deferred until it's a good moment to run it.
              */
-            char *dup = dupstr(p);     /* 'value' is not a const char * */
-            int retd = cmdline_process_param("-P", dup, 1, conf);
-            sfree(dup);
+            int retd = cmdline_process_param(
+                cmdline_arg_from_str(arg->list, "-P"), arg, 1, conf);
             assert(retd == 2);
             seen_port_argument = true;
             return 1;
@@ -453,7 +460,10 @@ int cmdline_process_param(const char *p, char *value,
         RETURN(2);
         UNAVAILABLE_IN(TOOLTYPE_NONNETWORK);
         SAVEABLE(0);
-        conf_set_str(conf, CONF_username, value);
+        if (value_utf8)
+            conf_set_utf8(conf, CONF_username, value_utf8);
+        else
+            conf_set_str(conf, CONF_username, value);
     }
     if (!strcmp(p, "-loghost")) {
         RETURN(2);
@@ -553,20 +563,23 @@ int cmdline_process_param(const char *p, char *value,
         sfree(host);
     }
     if (!strcmp(p, "-m")) {
-        const char *filename;
+        Filename *filename;
         FILE *fp;
 
         RETURN(2);
         UNAVAILABLE_IN(TOOLTYPE_FILETRANSFER | TOOLTYPE_NONNETWORK);
         SAVEABLE(0);
 
-        filename = value;
+        filename = cmdline_arg_to_filename(nextarg);
 
-        fp = fopen(filename, "r");
+        fp = f_open(filename, "r", false);
         if (!fp) {
-            cmdline_error("unable to open command file \"%s\"", filename);
+            cmdline_error("unable to open command file \"%s\"",
+                          filename_to_str(filename));
+            filename_free(filename);
             return ret;
         }
+        filename_free(filename);
         strbuf *command = strbuf_new();
         char readbuf[4096];
         while (1) {
@@ -603,11 +616,9 @@ int cmdline_process_param(const char *p, char *value,
             }
 
             cmdline_password = dupstr(value);
-            /* Assuming that `value' is directly from argv, make a good faith
-             * attempt to trample it, to stop it showing up in `ps' output
-             * on Unix-like systems. Not guaranteed, of course. */
-            smemclr(value, strlen(value));
         }
+
+        cmdline_arg_wipe(nextarg);
     }
 
     if (!strcmp(p, "-pwfile")) {
@@ -620,7 +631,7 @@ int cmdline_process_param(const char *p, char *value,
             cmdline_error("the -pwfile option can only be used with the "
                           "SSH protocol");
         else {
-            Filename *fn = filename_from_str(value);
+            Filename *fn = cmdline_arg_to_filename(nextarg);
             FILE *fp = f_open(fn, "r", false);
             if (!fp) {
                 cmdline_error("unable to open password file '%s'", value);
@@ -742,21 +753,19 @@ int cmdline_process_param(const char *p, char *value,
     }
 
     if (!strcmp(p, "-i")) {
-        Filename *fn;
         RETURN(2);
         UNAVAILABLE_IN(TOOLTYPE_NONNETWORK);
         SAVEABLE(0);
-        fn = filename_from_str(value);
+        Filename *fn = cmdline_arg_to_filename(nextarg);
         conf_set_filename(conf, CONF_keyfile, fn);
         filename_free(fn);
     }
 
     if (!strcmp(p, "-cert")) {
-        Filename *fn;
         RETURN(2);
         UNAVAILABLE_IN(TOOLTYPE_NONNETWORK);
         SAVEABLE(0);
-        fn = filename_from_str(value);
+        Filename *fn = cmdline_arg_to_filename(nextarg);
         conf_set_filename(conf, CONF_detached_cert, fn);
         filename_free(fn);
     }
@@ -774,7 +783,7 @@ int cmdline_process_param(const char *p, char *value,
         conf_set_int(conf, CONF_addressfamily, ADDRTYPE_IPV6);
     }
     if (!strcmp(p, "-sercfg")) {
-        char* nextitem;
+        const char *nextitem;
         RETURN(2);
         UNAVAILABLE_IN(TOOLTYPE_FILETRANSFER | TOOLTYPE_NONNETWORK);
         SAVEABLE(1);
@@ -791,7 +800,6 @@ int cmdline_process_param(const char *p, char *value,
                 skip = 0;
             } else {
                 length = end - nextitem;
-                nextitem[length] = '\0';
                 skip = 1;
             }
             if (length == 1) {
@@ -846,7 +854,9 @@ int cmdline_process_param(const char *p, char *value,
                 /* Messy special case */
                 conf_set_int(conf, CONF_serstopbits, 3);
             } else {
-                int serspeed = atoi(nextitem);
+                char *speedstr = dupprintf("%.*s", length, nextitem);
+                int serspeed = atoi(speedstr);
+                sfree(speedstr);
                 if (serspeed != 0) {
                     conf_set_int(conf, CONF_serspeed, serspeed);
                 } else {
@@ -859,12 +869,11 @@ int cmdline_process_param(const char *p, char *value,
     }
 
     if (!strcmp(p, "-sessionlog")) {
-        Filename *fn;
         RETURN(2);
         UNAVAILABLE_IN(TOOLTYPE_FILETRANSFER);
         /* but available even in TOOLTYPE_NONNETWORK, cf pterm "-log" */
         SAVEABLE(0);
-        fn = filename_from_str(value);
+        Filename *fn = cmdline_arg_to_filename(nextarg);
         conf_set_filename(conf, CONF_logfilename, fn);
         conf_set_int(conf, CONF_logtype, LGTYP_DEBUG);
         filename_free(fn);
@@ -872,11 +881,10 @@ int cmdline_process_param(const char *p, char *value,
 
     if (!strcmp(p, "-sshlog") ||
         !strcmp(p, "-sshrawlog")) {
-        Filename *fn;
         RETURN(2);
         UNAVAILABLE_IN(TOOLTYPE_NONNETWORK);
         SAVEABLE(0);
-        fn = filename_from_str(value);
+        Filename *fn = cmdline_arg_to_filename(nextarg);
         conf_set_filename(conf, CONF_logfilename, fn);
         conf_set_int(conf, CONF_logtype,
                      !strcmp(p, "-sshlog") ? LGTYP_PACKETS :
@@ -906,6 +914,33 @@ int cmdline_process_param(const char *p, char *value,
         conf_set_str(conf, CONF_proxy_telnet_command, value);
     }
 
+    if (!strcmp(p, "-batch")) {
+        RETURN(1);
+        SAVEABLE(0);
+        if (!console_set_batch_mode(true)) {
+            cmdline_report_unavailable(p);
+            return ret;
+        }
+    }
+
+    if (!strcmp(p, "-legacy-stdio-prompts") ||
+        !strcmp(p, "-legacy_stdio_prompts")) {
+        RETURN(1);
+        if (!console_set_stdio_prompts(true)) {
+            cmdline_report_unavailable(p);
+            return ret;
+        }
+    }
+
+    if (!strcmp(p, "-legacy-charset-handling") ||
+        !strcmp(p, "-legacy_charset_handling")) {
+        RETURN(1);
+        if (!set_legacy_charset_handling(true)) {
+            cmdline_report_unavailable(p);
+            return ret;
+        }
+    }
+
 #ifdef _WINDOWS
     /*
      * Cross-tool options only available on Windows.
@@ -923,12 +958,9 @@ int cmdline_process_param(const char *p, char *value,
 void cmdline_run_saved(Conf *conf)
 {
     for (size_t pri = 0; pri < NPRIORITIES; pri++) {
-        for (size_t i = 0; i < saves[pri].nsaved; i++) {
+        for (size_t i = 0; i < saves[pri].nsaved; i++)
             cmdline_process_param(saves[pri].params[i].p,
                                   saves[pri].params[i].value, 0, conf);
-            sfree(saves[pri].params[i].p);
-            sfree(saves[pri].params[i].value);
-        }
         saves[pri].nsaved = 0;
     }
 }
