@@ -50,7 +50,7 @@ struct psocks_state {
     unsigned log_flags;
     RecordDestination rec_dest;
     char *rec_cmd;
-    strbuf *subcmd;
+    bool got_subcmd;
 
     ConnectionLayer cl;
 };
@@ -72,7 +72,7 @@ struct psocks_connection {
 
 static SshChannel *psocks_lportfwd_open(
     ConnectionLayer *cl, const char *hostname, int port,
-    const char *description, const SocketPeerInfo *pi, Channel *chan);
+    const char *description, const SocketEndpointInfo *pi, Channel *chan);
 
 static const ConnectionLayerVtable psocks_clvt = {
     .lportfwd_open = psocks_lportfwd_open,
@@ -93,8 +93,9 @@ static const SshChannelVtable psocks_scvt = {
     /* all the rest are NULL */
 };
 
-static void psocks_plug_log(Plug *p, PlugLogType type, SockAddr *addr,
-                            int port, const char *error_msg, int error_code);
+static void psocks_plug_log(Plug *p, Socket *s, PlugLogType type,
+                            SockAddr *addr, int port,
+                            const char *error_msg, int error_code);
 static void psocks_plug_closing(Plug *p, PlugCloseType, const char *error_msg);
 static void psocks_plug_receive(Plug *p, int urgent,
                                 const char *data, size_t len);
@@ -154,7 +155,7 @@ static void psocks_connection_establish(void *vctx);
 
 static SshChannel *psocks_lportfwd_open(
     ConnectionLayer *cl, const char *hostname, int port,
-    const char *description, const SocketPeerInfo *pi, Channel *chan)
+    const char *description, const SocketEndpointInfo *pi, Channel *chan)
 {
     psocks_state *ps = container_of(cl, psocks_state, cl);
     psocks_connection *conn = snew(psocks_connection);
@@ -320,8 +321,9 @@ static void psocks_sc_unthrottle(SshChannel *sc, size_t bufsize)
 	sk_set_frozen(conn->socket, false);
 }
 
-static void psocks_plug_log(Plug *plug, PlugLogType type, SockAddr *addr,
-                            int port, const char *error_msg, int error_code)
+static void psocks_plug_log(Plug *plug, Socket *s, PlugLogType type,
+                            SockAddr *addr, int port,
+                            const char *error_msg, int error_code)
 {
     psocks_connection *conn = container_of(plug, psocks_connection, plug);
     char addrbuf[256];
@@ -407,7 +409,6 @@ psocks_state *psocks_new(const PsocksPlatform *platform)
     ps->log_flags = LOG_CONNSTATUS;
     ps->rec_dest = REC_NONE;
     ps->platform = platform;
-    ps->subcmd = strbuf_new();
 
     return ps;
 }
@@ -415,19 +416,19 @@ psocks_state *psocks_new(const PsocksPlatform *platform)
 void psocks_free(psocks_state *ps)
 {
     portfwdmgr_free(ps->portfwdmgr);
-    strbuf_free(ps->subcmd);
     sfree(ps->rec_cmd);
     sfree(ps);
 }
 
-void psocks_cmdline(psocks_state *ps, int argc, char **argv)
+void psocks_cmdline(psocks_state *ps, CmdlineArgList *arglist)
 {
     bool doing_opts = true;
-    bool accumulating_exec_args = false;
+    size_t arglistpos = 0;
     size_t args_seen = 0;
 
-    while (--argc > 0) {
-	const char *p = *++argv;
+    while (arglist->args[arglistpos]) {
+	CmdlineArg *arg = arglist->args[arglistpos++];
+        const char *p = cmdline_arg_to_str(arg);
 
 	if (doing_opts && p[0] == '-' && p[1]) {
             if (!strcmp(p, "--")) {
@@ -444,8 +445,9 @@ void psocks_cmdline(psocks_state *ps, int argc, char **argv)
                             "platform\n");
 		    exit(1);
                 }
-		if (--argc > 0) {
-		    ps->rec_cmd = dupstr(*++argv);
+		if (arglist->args[arglistpos] > 0) {
+		    ps->rec_cmd = dupstr(
+                        cmdline_arg_to_str(arglist->args[arglistpos++]));
 		} else {
 		    fprintf(stderr, "psocks: expected an argument to '-p'\n");
 		    exit(1);
@@ -457,10 +459,15 @@ void psocks_cmdline(psocks_state *ps, int argc, char **argv)
                             "supported on this platform\n");
 		    exit(1);
                 }
-                accumulating_exec_args = true;
+                if (!arglist->args[arglistpos]) {
+		    fprintf(stderr, "psocks: --exec requires a command\n");
+		    exit(1);
+                }
                 /* Now consume all further argv words for the
                  * subcommand, even if they look like options */
-                doing_opts = false;
+                ps->platform->found_subcommand(arglist->args[arglistpos]);
+                ps->got_subcmd = true;
+                break;
 	    } else if (!strcmp(p, "--help")) {
                 printf("usage: psocks [ -d ] [ -f");
                 if (ps->platform->open_pipes)
@@ -488,9 +495,7 @@ void psocks_cmdline(psocks_state *ps, int argc, char **argv)
                 exit(1);
             }
 	} else {
-            if (accumulating_exec_args) {
-                put_asciz(ps->subcmd, p);
-            } else switch (args_seen++) {
+            switch (args_seen++) {
               case 0:
                 ps->listen_port = atoi(p);
                 break;
@@ -513,8 +518,8 @@ void psocks_start(psocks_state *ps)
 
     portfwdmgr_config(ps->portfwdmgr, conf);
 
-    if (ps->subcmd->len)
-        ps->platform->start_subcommand(ps->subcmd);
+    if (ps->got_subcmd)
+        ps->platform->start_subcommand();
 
     conf_free(conf);
 }
@@ -529,7 +534,7 @@ int check_stored_host_key(const char *hostname, int port,
     unreachable("host keys not handled in this tool");
 }
 
-void store_host_key(const char *hostname, int port,
+void store_host_key(Seat *seat, const char *hostname, int port,
                     const char *keytype, const char *key)
 {
     unreachable("host keys not handled in this tool");
