@@ -1,9 +1,6 @@
 /*
- * Internals of the Terminal structure, for those other modules
- * which need to look inside it. It would be nice if this could be
- * folded back into terminal.c in future, with an abstraction layer
- * to handle everything that other modules need to know about it;
- * but for the moment, this will do.
+ * Internals of the Terminal structure, used by other modules in the
+ * terminal subdirectory and by test suites.
  */
 
 #ifndef PUTTY_TERMINAL_H
@@ -74,6 +71,8 @@ struct term_utf8_decode {
     int size;                          /* The size of the UTF character. */
 };
 
+struct term_userpass_state;
+
 struct terminal_tag {
 
     int compatibility_level;
@@ -158,6 +157,7 @@ struct terminal_tag {
     int raw_mouse_reported_y;
 
     bool bracketed_paste, bracketed_paste_active;
+    bool no_bracketed_paste;           /* disabled in configuration */
 
     int cset_attr[2];
 
@@ -227,7 +227,7 @@ struct terminal_tag {
     int attr_mask;
 
     wchar_t *paste_buffer;
-    int paste_len, paste_pos;
+    size_t paste_len, paste_pos;
 
     Backend *backend;
 
@@ -302,8 +302,7 @@ struct terminal_tag {
      * the former every time.
      */
     bool ansi_colour;
-    char *answerback;
-    int answerbacklen;
+    strbuf *answerback;
     bool no_arabicshaping;
     int beep;
     bool bellovl;
@@ -429,11 +428,21 @@ struct terminal_tag {
         WIN_RESIZE_NO, WIN_RESIZE_NEED_SEND, WIN_RESIZE_AWAIT_REPLY
     } win_resize_pending;
     int win_resize_pending_w, win_resize_pending_h;
+
+    /*
+     * Indicates whether term_get_userpass_input is currently using
+     * the terminal to present a password prompt or similar, and if
+     * so, whether it's overridden the terminal into UTF-8 mode.
+     */
+    struct term_userpass_state *userpass_state;
+    bool userpass_utf8_override;
 };
 
 static inline bool in_utf(Terminal *term)
 {
-    return term->utf || term->ucsdata->line_codepage == CP_UTF8;
+    return (term->utf ||
+            term->ucsdata->line_codepage == CP_UTF8 ||
+            (term->userpass_state && term->userpass_utf8_override));
 }
 
 unsigned long term_translate(
@@ -562,4 +571,47 @@ static inline bool decpos_fn(pos *p, int cols)
 #define incpos(p) incpos_fn(&(p), GET_TERM_COLS)
 #define decpos(p) decpos_fn(&(p), GET_TERM_COLS)
 
+struct TermLineEditorCallbackReceiverVtable {
+    void (*to_terminal)(TermLineEditorCallbackReceiver *rcv, ptrlen data);
+    void (*to_backend)(TermLineEditorCallbackReceiver *rcv, ptrlen data);
+    void (*special)(TermLineEditorCallbackReceiver *rcv,
+                    SessionSpecialCode code, int arg);
+    void (*newline)(TermLineEditorCallbackReceiver *rcv);
+};
+struct TermLineEditorCallbackReceiver {
+    const TermLineEditorCallbackReceiverVtable *vt;
+};
+TermLineEditor *lineedit_new(Terminal *term, unsigned flags,
+                             TermLineEditorCallbackReceiver *receiver);
+void lineedit_free(TermLineEditor *le);
+void lineedit_input(TermLineEditor *le, char ch, bool dedicated);
+void lineedit_modify_flags(TermLineEditor *le, unsigned clr, unsigned flip);
+void lineedit_send_line(TermLineEditor *le);
+
+/*
+ * Flags controlling the behaviour of TermLineEditor.
+ */
+#define LINEEDIT_FLAGS(X) \
+    X(LE_INTERRUPT)           /* pass SS_IP back to client on ^C */ \
+    X(LE_SUSPEND)             /* pass SS_SUSP back to client on ^Z */ \
+    X(LE_ABORT)               /* pass SS_ABORT back to client on ^\ */ \
+    X(LE_EOF_ALWAYS)          /* pass SS_EOF to client on *any* ^D \
+                               * (not just if the line buffer is empty) */ \
+    X(LE_ESC_ERASES)          /* make ESC erase the line, as well as ^U */ \
+    X(LE_CRLF_NEWLINE)        /* interpret manual ^M^J the same as Return */ \
+    /* end of list */
+enum {
+    #define ALLOCATE_BIT_POSITION(flag) flag ## _bitpos,
+    LINEEDIT_FLAGS(ALLOCATE_BIT_POSITION)
+    #undef ALLOCATE_BIT_POSITION
+};
+enum {
+    #define DEFINE_FLAG_BIT(flag) flag = 1 << flag ## _bitpos,
+    LINEEDIT_FLAGS(DEFINE_FLAG_BIT)
+    #undef DEFINE_FLAG_BIT
+};
+
+termline *term_get_line(Terminal *term, int y);
+void term_release_line(termline *line);
+
 #endif
