@@ -84,10 +84,6 @@ typedef void *HelpCtx;
 #define NULL_HELPCTX ((HelpCtx)NULL)
 #define HELPCTX(x) NULL
 
-typedef const char *FILESELECT_FILTER_TYPE;
-#define FILTER_KEY_FILES NULL          /* FIXME */
-#define FILTER_DYNLIB_FILES NULL       /* FIXME */
-
 /*
  * Under X, selection data must not be NUL-terminated.
  */
@@ -398,13 +394,23 @@ void setup_fd_socket(Socket *s, int infd, int outfd, int inerrfd);
 void fd_socket_set_psb_prefix(Socket *s, const char *prefix);
 
 /*
- * Default font setting, which can vary depending on NOT_X_WINDOWS.
+ * Default font settings. We have a default font for each of
+ * client-side and server-side, so that we can use one of each as a
+ * fallback, and we also have a single overall default which goes into
+ * Conf to populate the initial state of Default Settings.
+ *
+ * In the past, this default varied with NOT_X_WINDOWS. But these days
+ * non-X11 environments like Wayland with only client-side fonts are
+ * common, and even an X11-capable _build_ of PuTTY is quite likely to
+ * find out at run time that X11 and its bitmap fonts aren't
+ * available. Also, a fixed-size bitmap font doesn't play nicely with
+ * high-DPI displays. And the GTK1 build of PuTTY, which can _only_
+ * handle server-side fonts, is legacy. So the default font is
+ * unconditionally the client-side one.
  */
-#ifdef NOT_X_WINDOWS
-#define DEFAULT_GTK_FONT "client:Monospace 12"
-#else
-#define DEFAULT_GTK_FONT "server:fixed"
-#endif
+#define DEFAULT_GTK_CLIENT_FONT "client:Monospace 12"
+#define DEFAULT_GTK_SERVER_FONT "server:fixed"
+#define DEFAULT_GTK_FONT DEFAULT_GTK_CLIENT_FONT
 
 /*
  * pty.c.
