@@ -451,34 +451,6 @@ static INT_PTR CALLBACK PPKParamsProc(HWND hwnd, UINT msg,
     return 0;
 }
 
-/*
- * Prompt for a key file. Assumes the filename buffer is of size
- * FILENAME_MAX.
- */
-static bool prompt_keyfile(HWND hwnd, char *dlgtitle,
-                           char *filename, bool save, bool ppk)
-{
-    OPENFILENAME of;
-    memset(&of, 0, sizeof(of));
-    of.hwndOwner = hwnd;
-    if (ppk) {
-        of.lpstrFilter = "PuTTY Private Key Files (*.ppk)\0*.ppk\0"
-            "All Files (*.*)\0*\0\0\0";
-        of.lpstrDefExt = ".ppk";
-    } else {
-        of.lpstrFilter = "All Files (*.*)\0*\0\0\0";
-    }
-    of.lpstrCustomFilter = NULL;
-    of.nFilterIndex = 1;
-    of.lpstrFile = filename;
-    *filename = '\0';
-    of.nMaxFile = FILENAME_MAX;
-    of.lpstrFileTitle = NULL;
-    of.lpstrTitle = dlgtitle;
-    of.Flags = 0;
-    return request_file(NULL, &of, false, save);
-}
-
 /*
  * Dialog-box function for the Licence box.
  */
@@ -2016,7 +1988,6 @@ static INT_PTR CALLBACK MainDlgProc(HWND hwnd, UINT msg,
             state =
                 (struct MainDlgState *) GetWindowLongPtr(hwnd, GWLP_USERDATA);
             if (state->key_exists) {
-                char filename[FILENAME_MAX];
                 char *passphrase, *passphrase2;
                 int type, realtype;
 
@@ -2068,26 +2039,28 @@ static INT_PTR CALLBACK MainDlgProc(HWND hwnd, UINT msg,
                         break;
                     }
                 }
-                if (prompt_keyfile(hwnd, "Save private key as:",
-                                   filename, true, (type == realtype))) {
+                Filename *fn = request_file(
+                    hwnd, "Save private key as:", NULL, true, NULL, false,
+                    (type==realtype ? FILTER_KEY_FILES : FILTER_ALL_FILES));
+                if (fn) {
                     int ret;
-                    FILE *fp = fopen(filename, "r");
+                    FILE *fp = f_open(fn, "r", false);
                     if (fp) {
                         char *buffer;
                         fclose(fp);
                         buffer = dupprintf("Overwrite existing file\n%s?",
-                                           filename);
+                                           filename_to_str(fn));
                         ret = MessageBox(hwnd, buffer, "PuTTYgen Warning",
                                          MB_YESNO | MB_ICONWARNING);
                         sfree(buffer);
                         if (ret != IDYES) {
                             burnstr(passphrase);
+                            filename_free(fn);
                             break;
                         }
                     }
 
                     if (state->ssh2) {
-                        Filename *fn = filename_from_str(filename);
                         if (type != realtype)
                             ret = export_ssh2(fn, type, &state->ssh2key,
                                               *passphrase ? passphrase : NULL);
@@ -2095,21 +2068,19 @@ static INT_PTR CALLBACK MainDlgProc(HWND hwnd, UINT msg,
                             ret = ppk_save_f(fn, &state->ssh2key,
                                              *passphrase ? passphrase : NULL,
                                              &save_params);
-                        filename_free(fn);
                     } else {
-                        Filename *fn = filename_from_str(filename);
                         if (type != realtype)
                             ret = export_ssh1(fn, type, &state->key,
                                               *passphrase ? passphrase : NULL);
                         else
                             ret = rsa1_save_f(fn, &state->key,
                                               *passphrase ? passphrase : NULL);
-                        filename_free(fn);
                     }
                     if (ret <= 0) {
                         MessageBox(hwnd, "Unable to save key file",
                                    "PuTTYgen Error", MB_OK | MB_ICONERROR);
                     }
+                    filename_free(fn);
                 }
                 burnstr(passphrase);
             }
@@ -2120,23 +2091,26 @@ static INT_PTR CALLBACK MainDlgProc(HWND hwnd, UINT msg,
             state =
                 (struct MainDlgState *) GetWindowLongPtr(hwnd, GWLP_USERDATA);
             if (state->key_exists) {
-                char filename[FILENAME_MAX];
-                if (prompt_keyfile(hwnd, "Save public key as:",
-                                   filename, true, false)) {
+                Filename *fn = request_file(
+                    hwnd, "Save public key as:", NULL, true, NULL, false,
+                    FILTER_ALL_FILES);
+                if (fn) {
                     int ret;
-                    FILE *fp = fopen(filename, "r");
+                    FILE *fp = f_open(fn, "r", false);
                     if (fp) {
                         char *buffer;
                         fclose(fp);
                         buffer = dupprintf("Overwrite existing file\n%s?",
-                                           filename);
+                                           filename_to_str(fn));
                         ret = MessageBox(hwnd, buffer, "PuTTYgen Warning",
                                          MB_YESNO | MB_ICONWARNING);
                         sfree(buffer);
-                        if (ret != IDYES)
+                        if (ret != IDYES) {
+                            filename_free(fn);
                             break;
+                        }
                     }
-                    fp = fopen(filename, "w");
+                    fp = f_open(fn, "w", false);
                     if (!fp) {
                         MessageBox(hwnd, "Unable to open key file",
                                    "PuTTYgen Error", MB_OK | MB_ICONERROR);
@@ -2157,6 +2131,7 @@ static INT_PTR CALLBACK MainDlgProc(HWND hwnd, UINT msg,
                                        "PuTTYgen Error", MB_OK | MB_ICONERROR);
                         }
                     }
+                    filename_free(fn);
                 }
             }
             break;
@@ -2167,10 +2142,11 @@ static INT_PTR CALLBACK MainDlgProc(HWND hwnd, UINT msg,
             state =
                 (struct MainDlgState *) GetWindowLongPtr(hwnd, GWLP_USERDATA);
             if (!state->generation_thread_exists) {
-                char filename[FILENAME_MAX];
-                if (prompt_keyfile(hwnd, "Load private key:", filename, false,
-                                   LOWORD(wParam) == IDC_LOAD)) {
-                    Filename *fn = filename_from_str(filename);
+                Filename *fn = request_file(
+                    hwnd, "Load private key:", NULL, false, NULL, false,
+                    (LOWORD(wParam) == IDC_LOAD ?
+                     FILTER_KEY_FILES : FILTER_ALL_FILES));
+                if (fn) {
                     load_key_file(hwnd, state, fn, LOWORD(wParam) != IDC_LOAD);
                     filename_free(fn);
                 }
@@ -2182,10 +2158,10 @@ static INT_PTR CALLBACK MainDlgProc(HWND hwnd, UINT msg,
             state =
                 (struct MainDlgState *) GetWindowLongPtr(hwnd, GWLP_USERDATA);
             if (state->key_exists && !state->generation_thread_exists) {
-                char filename[FILENAME_MAX];
-                if (prompt_keyfile(hwnd, "Load certificate:", filename, false,
-                                   false)) {
-                    Filename *fn = filename_from_str(filename);
+                Filename *fn = request_file(
+                    hwnd, "Load certificate:", NULL, false, NULL, false,
+                    FILTER_ALL_FILES);
+                if (fn) {
                     add_certificate(hwnd, state, fn);
                     filename_free(fn);
                 }
@@ -2394,6 +2370,7 @@ int WINAPI WinMain(HINSTANCE inst, HINSTANCE prev, LPSTR cmdline, int show)
     struct InitialParams params[1];
 
     dll_hijacking_protection();
+    enable_dit();
 
     init_common_controls();
     hinst = inst;
