@@ -1,9 +1,11 @@
 /*
- * Handy wrapper around GetDlgItemText which doesn't make you invent
- * an arbitrary length limit on the output string. Returned string is
- * dynamically allocated; caller must free.
+ * Handy wrappers around GetDlgItemText (A and W) which don't make you
+ * invent an arbitrary length limit on the output string. Returned
+ * string is dynamically allocated; caller must free.
  */
 
+#include <wchar.h>
+
 #include "putty.h"
 
 char *GetDlgItemText_alloc(HWND hwnd, int id)
@@ -18,3 +20,16 @@ char *GetDlgItemText_alloc(HWND hwnd, int id)
 
     return ret;
 }
+
+wchar_t *GetDlgItemTextW_alloc(HWND hwnd, int id)
+{
+    wchar_t *ret = NULL;
+    size_t size = 0;
+
+    do {
+        sgrowarray_nm(ret, size, size);
+        GetDlgItemTextW(hwnd, id, ret, size);
+    } while (!wmemchr(ret, L'\0', size-1));
+
+    return ret;
+}
